# CDMA Address Width 64 bits, Data Width 32 bits, Burst Size 16 bits
# PS + AXI-Slave <-> CDMA M_AXI

from pynq import Overlay
import numpy as np
import pynq.lib.dma
from pynq_cdma import CDMA
import time

ol = Overlay("./dma.bit")

# print(ol.ip_dict)
print("==============================dir(ol)================================","\n")
print(dir(ol),"\n")
print("==============================dir(ddr4_ip)================================","\n")
ddr4_ip = ol.ddr4_0
print(dir(ddr4_ip),"\n")
print("==============================dir(ddr4_ip.mmio.array)================================","\n")
print(dir(ddr4_ip.mmio.array),"\n")

axi_cdma_ip = ol.axi_cdma_0
print("==============================dir(axi_cdma_ip)================================","\n")
print(dir(axi_cdma_ip),"\n")
print("==============================axi_cdma_ip.register_map================================","\n")
print(axi_cdma_ip.register_map, "\n")


# num = 10*1024*1024
num = 16

ps_dram_buffer = pynq.allocate(shape=(num,), dtype=np.uint8)
print(dir(ps_dram_buffer))
print("ps_dram_buffer.physical_address", ps_dram_buffer.physical_address)
print("ps_dram_buffer.physical_address", hex(ps_dram_buffer.physical_address))
print("ps_dram_buffer.device_address", ps_dram_buffer.device_address)
# pl_dram_buffer = pynq.allocate(shape=(num,), dtype=np.uint32)
# print("pl_dram_buffer.physical_address", pl_dram_buffer.physical_address)

# Get the address of the PL DRAM from the overlay
# pl_dram_address = ol.ip_dict['axi_cdma_0']['phys_addr'] + 0x1000
# pl_dram_address = 0x500000000 + 0x1000
pl_dram_base_address = 0x05_0000_0000
pl_dram_offset_address = 0x1000
pl_dram_address = pl_dram_base_address + pl_dram_offset_address

# Allocate buffer in PL DRAM
# pl_dram_buffer = pynq.allocate(shape=(num,), dtype=np.uint32, physical_address=pl_dram_address)

# for i in range(3):
#     ps_dram_buffer[i] = i
ps_dram_buffer[:] = np.arange(num)

psddr_value = ol.PSDDR.read(ps_dram_buffer.physical_address)
print("psddr_value", hex(psddr_value))

print("PS DRAM buffer:", ps_dram_buffer)
print("ps_dram_buffer.nbytes = ", ps_dram_buffer.nbytes /1024/1024, " MB ")
print("PL DRAM buffer:")
pl_dram_buffer = np.empty(num, dtype=np.uint32)
for i in range(num):
    pl_dram_buffer[i] = ddr4_ip.read(pl_dram_offset_address+i*4)
print(pl_dram_buffer)
print("============================ start ============================")
start = time.time()

# axi_cdma_ip.transfer(ps_dram_buffer, pl_dram_address, bytes_count=num * 4) # 2034237440, 4026531840
# axi_cdma_ip.transfer(0x0000_000F, output_buffer)

# Configure CDMA to transfer from PL DRAM to PS DRAM

# CDMASR.Idle = 0 : Not Idle – Simple or SG DMA operations are in progress
# CDMASR.Idle = 1 : Idle – Simple or SG operations completed or not started

axi_cdma_ip.write(0x00, 0x4)  # Set the CDMA to Simple Mode
axi_cdma_ip.write(0x18, ps_dram_buffer.physical_address)  # Set the source address
axi_cdma_ip.write(0x20, pl_dram_address)  # Set the destination address
axi_cdma_ip.write(0x24, pl_dram_address >> 32)  # Set the destination address MSB

# print(axi_cdma_ip.register_map)
axi_cdma_ip.write(0x28, num)  # Set the transfer length (in bytes)
print(axi_cdma_ip.register_map)

# Wait for the transfer to complete
while (axi_cdma_ip.read(0x04) & 0x2) == 0:
    pass

print(axi_cdma_ip.register_map)

end = time.time()

print("============================ end ============================")

# output_array = output_buffer.copy()

print("PL DRAM buffer:")
pl_dram_buffer = np.empty(num, dtype=np.uint32)
for i in range(num):
    pl_dram_buffer[i] = ddr4_ip.read(pl_dram_offset_address+i*4)
print(pl_dram_buffer)

# print(ddr4_ip.mmio.array.max)

time_cost = end - start

print("time cost: {:.5f} s".format(time_cost))
length = pl_dram_buffer.nbytes
cost = round(end - start, 5)
speed = round(length/(cost*1048576), 5)
print("transfer speed: " + str(speed) + "MB/s")

print(axi_cdma_ip.register_map)

